#!/bin/bash
INTEL_UTIL="./eeupdate64e"
T_NIC_1="I225"
T_FW_FILE_1="FW/I225V_FXVL_15F3_V_1MB_1.82_20220420.bin"
T_NIC_2="E3100"
T_FW_FILE_2="FW/E3100_FXVL_3100X_K_1MB_1.82_20230706.bin"
T_NIC_FW="1.82"
G_UPDATE=0
G_LOG_FILE="/tmp/auto_nic_update.log"
G_INPUT=N

#$1=Dev No, $2=FW file.
function func_update_nic_fw {
    #log execute cmd
    date >> ${G_LOG_FILE}
    echo "${INTEL_UTIL} /nic=${1} /d ${2}" >> ${G_LOG_FILE}
	#do update
    ret=$(${INTEL_UTIL} /nic=${1} /d ${2})
    #log result
    echo ${ret} >> ${G_LOG_FILE}
    #print result
    result=$(echo ${ret} |grep "updated successfully")
    if [ "${result}" = "" ]; then
        echo "    =>Updated failed!"
    else
        echo "    =>Updated successfully."
    fi
}

#$1=Target NIC name, $2=Target FW version.
function func_check_nic_fw {
    t_fw_major=$(echo "${2}" | cut -d "." -f1)
    t_fw_minor=$(echo "${2}" | cut -d "." -f2)
    nic_list=$(${INTEL_UTIL} | grep ${1} | awk '{print $1}')
    
    echo " "
    echo "Check NIC ${1}:"
    if [ "${nic_list}" = "" ]; then
        echo "  No device found!"
	fi
	
    for nic in ${nic_list}
    do
        nic_fw=$(${INTEL_UTIL} /nic=${nic} /eepromver | grep "Image Version" | awk '{print $5}')
        nic_fw_major=$(echo "${nic_fw}" | cut -d "." -f1)
        nic_fw_minor=$(echo "${nic_fw}" | cut -d "." -f2)
#echo "FW:${nic_fw_major}.${nic_fw_minor} , FW:${t_fw_major}.${t_fw_minor}"
        if [ "${nic_fw_major}" -lt "${t_fw_major}" ] || [ "${G_INPUT}" = "Y" ]; then
            result="FW updating..."
            G_UPDATE=1
        else
            if [ "${nic_fw_minor}" -lt "${t_fw_minor}" ]; then
                result="FW updating..."
                G_UPDATE=1
            else
                result="Pass"
            fi
        fi
        echo "  Dev:${nic}, NIC:${1}, FW:${nic_fw_major}.${nic_fw_minor}...${result}"

        if [ "${G_UPDATE}" = "1" ]; then
            if [ "${1}" = "${T_NIC_1}" ]; then
                func_update_nic_fw ${nic} ${T_FW_FILE_1}
            elif [ "${1}" = "${T_NIC_2}" ]; then
                func_update_nic_fw ${nic} ${T_FW_FILE_2}
			else
                echo "    =>Error! No available FW for this device."
			fi
        fi
    done
}

function func_main {
    #check execute result
    if [ -f ${G_LOG_FILE} ]; then
        check_exe=$(cat ${G_LOG_FILE} |grep "cold boot the NAS")
        if [ "${check_exe}" != "" ]; then
            echo "The auto-update tool had executed done."
            echo "Please cold boot the NAS to reload the new NIC FW!!!"
            return
        fi
    fi
    #print warning
    echo " "
    echo "Warning!!!"
	echo "This is QNAP NIC FW auto-update tool."
	echo "It's only for Intel I225 & E3100 NICs,"
    echo "force updating old FW to V${T_NIC_FW}."
    echo "Do you need to execute this FW update? N/y"
    read G_INPUT
    #execute tool
    if [ "${G_INPUT}" = "y" ] || [ "${G_INPUT}" = "Y" ]; then
        func_check_nic_fw ${T_NIC_1} ${T_NIC_FW}
	    func_check_nic_fw ${T_NIC_2} ${T_NIC_FW}
        echo " "
        echo "Auto-update tool is executing done."
        if [ "${G_UPDATE}" = "1" ]; then
            #log
            echo "Please cold boot the NAS to reload the new NIC FW!!!" >> ${G_LOG_FILE}
            echo "Please cold boot the NAS to reload the new NIC FW!!!"
        fi
    fi
    echo " "
}

func_main